@@ -158,7 +158,7 @@ class NamespaceDict(dict):
     A dictionary that can access its string keys through the namespace
     """
 
-    def __init__(self, seq: Mapping = None, **kwargs):
+    def __init__(self, *args, **kwargs):
         """
         NamespaceDict() -> new empty namespaced dictionary
         NamespaceDict(mapping) -> new namespaced dictionary initialized from a mapping object's
@@ -170,29 +170,52 @@ class NamespaceDict(dict):
         NamespaceDict(**kwargs) -> new namespaced dictionary initialized with the name=value pairs
             in the keyword argument list.  For example:  NamespaceDict(one=1, two=2)
         """
-        if seq:
-            super().__init__(seq)
+        super().__init__(*args, **kwargs)
+        for key, value in self.items():
+            if isinstance(value, dict):
+                super().__setitem__(key, NamespaceDict(value))
+
+    def __setitem__(self, k, v):
+        if isinstance(v, Mapping):
+            v = NamespaceDict(v)
+        super().__setitem__(k, v)
+
+    def update(self, *args, **kwargs):
+        other = dict(*args, **kwargs)
+        for k, v in other.items():
+            self[k] = v
+
+    def __setattr__(self, k, v):
+        if k in self.__dict__ or k in self.__class__.__dict__:
+            # if the attribute already exists in the instance dictionary or in the class dictionary
+            super().__setattr__(k, v)
         else:
-            super().__init__()
-
-        self.update(**kwargs)
-
-    def __setattr__(self, k: str, v: Any):
-        self[k] = v
+            self[k] = v
 
     def __getattr__(self, k: str):
-        if hasattr(dict, k):
-            return super().__getattribute__(k)
-
         # Swap key errors for attribute errors so that we don't blow up on "hasattr" checks
-        try:
-            value = self[k]
-        except KeyError as e:
-            raise AttributeError(e)
+        if k not in self:
+            raise AttributeError(
+                f"'{self.__class__.__name__}' object has no attribute '{k}'"
+            )
+        return self[k]
 
-        # Allow chain-getting namespaced values
-        if isinstance(value, Mapping):
-            return NamespaceDict(value)
+    def __copy__(self):
+        return NamespaceDict(dict(**self).copy())
+
+    def __deepcopy__(self, memodict=None):
+        return NamespaceDict(copy.deepcopy(dict(self), memo=memodict))
+
+    def __getstate__(self):
+        return dict(self)
+
+    def __setstate__(self, state):
+        self.update(state)
+
+    def pop(self, key, *args):
+        value = super().pop(key, *args)
+        if isinstance(value, dict):
+            value = NamespaceDict(value)
         return value
 
 
@@ -426,7 +449,7 @@ def recursive_diff(
             append_old = list(old.keys())[min_length:]
             append_new = list(new.keys())[min_length:]
         # Compare ordered
-        for (key_old, key_new) in zip(old, new):
+        for key_old, key_new in zip(old, new):
             if key_old == key_new:
                 if key_old in ignore_keys:
                     del ret_old[key_old]
@@ -478,8 +501,11 @@ def recursive_diff(
                 else:
                     ret_old[key] = res["old"]
                     ret_new[key] = res["new"]
-            elif ignore_missing_keys and key in old:
-                del ret_old[key]
+            elif ignore_missing_keys:
+                if key in old:
+                    del ret_old[key]
+                if key in new:
+                    del ret_new[key]
         ret = {"old": ret_old, "new": ret_new} if ret_old or ret_new else {}
     elif isinstance(old, set) and isinstance(new, set):
         ret = {"old": old - new, "new": new - old} if old - new or new - old else {}
