@@ -14,6 +14,10 @@ class _SpecialForm:
         return self._getitem(self, parameters)
 
 
+class ComputedValue:
+    ...
+
+
 # Types for type-hinting, they cannot be used in casting
 @_SpecialForm
 def Computed(self, type_: Type):
@@ -32,7 +36,11 @@ def Computed(self, type_: Type):
       def function(arg1: typing.Computed[str]):
           ...
     """
-    return Union[type_, type(None)]
+    return Union[type_, ComputedValue]
+
+
+class SensitiveValue:
+    ...
 
 
 @_SpecialForm
@@ -50,4 +58,4 @@ def Sensitive(self, type_: Type):
       def function(arg1: typing.Sensitive[str]):
           ...
     """
-    return Union[type_, type(None)]
+    return Union[type_, SensitiveValue]
